<?php

declare(strict_types=1);

namespace Dedi\SyliusImportExportPlugin\Reader;

use Port\Reader\CountableReader;
use SeekableIterator;
use SplFileObject;

final class TxtReader implements CountableReader, SeekableIterator
{
    private array $columnHeaders = [];

    private ?int $headerRowNumber = null;

    private int $pointer = 0;

    private array $rows;

    public function __construct(
        SplFileObject $file,
        string $delimiter,
        ?string $enclosure,
        bool $trim,
        bool $utf8Encode = false,
        ?int $headerRowNumber = null
    ) {
        $handle = fopen($file->getPathname(), 'r');

        $this->rows = [];
        $i = 0;
        while (false !== ($data = fgetcsv($handle, null, $delimiter, $enclosure))) {
            if ($data === [null]) {
                continue;
            }
            if ($trim) {
                $data = array_map(fn ($value): string => trim($value), $data);
            }
            if ($utf8Encode) {
                $data = array_map(fn ($value): string => utf8_encode($value), $data);
            }
            $this->rows[$i] = $data;
            ++$i;
        }
        fclose($handle);

        if (null !== $headerRowNumber) {
            $this->setHeaderRowNumber($headerRowNumber);
        }
    }

    public function count(): int
    {
        $count = count($this->rows);
        if (null !== $this->headerRowNumber) {
            --$count;
        }

        return $count;
    }

    public function current(): ?array
    {
        $row = $this->rows[$this->pointer];

        if (\count($this->columnHeaders) > 0 && \count($this->columnHeaders) === \count($row)) {
            return array_combine(array_values($this->columnHeaders), $row);
        }

        return $row;
    }

    public function getColumnHeaders(): array
    {
        return $this->columnHeaders;
    }

    public function getRow($number): array
    {
        $this->seek($number);

        return $this->current();
    }

    public function key(): int
    {
        return $this->pointer;
    }

    public function next(): void
    {
        ++$this->pointer;
    }

    public function rewind(): void
    {
        if (null === $this->headerRowNumber) {
            $this->pointer = 0;
        } else {
            $this->pointer = $this->headerRowNumber + 1;
        }
    }

    public function seek($pointer): void
    {
        $this->pointer = $pointer;
    }

    public function setColumnHeaders(array $columnHeaders): self
    {
        $this->columnHeaders = $columnHeaders;

        return $this;
    }

    public function setHeaderRowNumber($rowNumber): self
    {
        $this->headerRowNumber = $rowNumber;
        $this->columnHeaders = $this->rows[$rowNumber];

        return $this;
    }

    public function valid(): bool
    {
        return isset($this->rows[$this->pointer]);
    }
}
